This commits introduces a optimization by avoiding calling the
same functions again. For example, `git stash push -u`
would call at some points the following functions:

 * `check_changes()` (inside `do_push_stash()`)
 * `do_create_stash()`, which calls: `check_changes()` and
`get_untracked_files()`

Note that `check_changes()` also calls `get_untracked_files()`.
So, `check_changes()` is called 2 times and `get_untracked_files()`
3 times.

The old function `check_changes()` now consists of two functions:
`get_untracked_files()` and `check_changes_tracked_files()`.

These are the call chains for `push` and `create`:

 * `push_stash()` -> `do_push_stash()` -> `do_create_stash()`

 * `create_stash()` -> `do_create_stash()`

To prevent calling the same functions over and over again,
`check_changes()` inside `do_create_stash()` is now placed
in the caller functions (`create_stash()` and `do_push_stash()`).
This way `check_changes()` and `get_untracked files()` are called
only one time.

https://public-inbox.org/git/20180818223329.gj11...@hank.intra.tgummerer.com/

Signed-off-by: Paul-Sebastian Ungureanu <ungureanupaulsebast...@gmail.com>
---
 builtin/stash.c | 50 ++++++++++++++++++++++++++++---------------------
 1 file changed, 29 insertions(+), 21 deletions(-)

diff --git a/builtin/stash.c b/builtin/stash.c
index e945c13c42..d2365ada2e 100644
--- a/builtin/stash.c
+++ b/builtin/stash.c
@@ -881,18 +881,18 @@ static int get_untracked_files(struct pathspec ps, int 
include_untracked,
 }
 
 /*
  * The return value of `check_changes()` can be:
  *
  * < 0 if there was an error
  * = 0 if there are no changes.
  * > 0 if there are changes.
  */
-static int check_changes(struct pathspec ps, int include_untracked)
+
+static int check_changes_tracked_files(struct pathspec ps)
 {
        int result;
        struct rev_info rev;
        struct object_id dummy;
-       struct strbuf out = STRBUF_INIT;
 
        /* No initial commit. */
        if (get_oid("HEAD", &dummy))
@@ -920,14 +920,26 @@ static int check_changes(struct pathspec ps, int 
include_untracked)
        if (diff_result_code(&rev.diffopt, result))
                return 1;
 
+       return 0;
+}
+
+/*
+ * The function will fill `untracked_files` with the names of untracked files
+ * It will return 1 if there were any changes and 0 if there were not.
+ */
+
+static int check_changes(struct pathspec ps, int include_untracked,
+                        struct strbuf *untracked_files)
+{
+       int ret = 0;
+       if (check_changes_tracked_files(ps))
+               ret = 1;
+
        if (include_untracked && get_untracked_files(ps, include_untracked,
-                                                    &out)) {
-               strbuf_release(&out);
-               return 1;
-       }
+                                                    untracked_files))
+               ret = 1;
 
-       strbuf_release(&out);
-       return 0;
+       return ret;
 }
 
 static int save_untracked_files(struct stash_info *info, struct strbuf *msg,
@@ -1138,7 +1150,7 @@ static int do_create_stash(struct pathspec ps, char 
**stash_msg,
                head_commit = lookup_commit(the_repository, &info->b_commit);
        }
 
-       if (!check_changes(ps, include_untracked)) {
+       if (!check_changes(ps, include_untracked, &untracked_files)) {
                ret = 1;
                goto done;
        }
@@ -1163,8 +1175,7 @@ static int do_create_stash(struct pathspec ps, char 
**stash_msg,
                goto done;
        }
 
-       if (include_untracked && get_untracked_files(ps, include_untracked,
-                                                    &untracked_files)) {
+       if (include_untracked) {
                if (save_untracked_files(info, &msg, untracked_files)) {
                        if (!quiet)
                                fprintf_ln(stderr, _("Cannot save "
@@ -1244,19 +1255,15 @@ static int create_stash(int argc, const char **argv, 
const char *prefix)
        stash_msg = stash_msg_buf.buf;
 
        memset(&ps, 0, sizeof(ps));
-       ret = do_create_stash(ps, &stash_msg, 0, 0, &info, NULL, 0);
+       if (!check_changes_tracked_files(ps))
+               return 0;
 
-       if (!ret)
+       if (!(ret = do_create_stash(ps, &stash_msg, 0, 0, &info, NULL, 0)))
                printf_ln("%s", oid_to_hex(&info.w_commit));
 
        strbuf_release(&stash_msg_buf);
        free(stash_msg);
-
-       /*
-        * ret can be 1 if there were no changes. In this case, we should
-        * not error out.
-        */
-       return ret < 0;
+       return ret;
 }
 
 static int do_push_stash(struct pathspec ps, char *stash_msg, int quiet,
@@ -1265,6 +1272,7 @@ static int do_push_stash(struct pathspec ps, char 
*stash_msg, int quiet,
        int ret = 0;
        struct stash_info info;
        struct strbuf patch = STRBUF_INIT;
+       struct strbuf untracked_files = STRBUF_INIT;
 
        if (patch_mode && keep_index == -1)
                keep_index = 1;
@@ -1299,7 +1307,7 @@ static int do_push_stash(struct pathspec ps, char 
*stash_msg, int quiet,
                goto done;
        }
 
-       if (!check_changes(ps, include_untracked)) {
+       if (!check_changes(ps, include_untracked, &untracked_files)) {
                if (!quiet)
                        printf_ln(_("No local changes to save"));
                goto done;
@@ -881,18 +881,18 @@ static int get_untracked_files(struct pathspec ps, int 
 
 /*
- * The return value of `check_changes()` can be:
+ * The return value of `check_changes_tracked_files()` can be:
  *
  * < 0 if there was an error
-- 
2.19.0.rc0.23.g10a62394e7

Reply via email to